@@ -24,6 +24,7 @@
  */
 
 #include <gnumeric-config.h>
+#include <application.h>
 #include <gnumeric.h>
 #include <gnm-plugin.h>
 #include <gnumeric-conf.h>
@@ -89,7 +90,7 @@ go_gnm_component_get_data (GOComponent *component, gpointer *data, int *length,
 {
 	GOGnmComponent *gognm = GO_GNM_COMPONENT (component);
 	if (gognm->wv) {
-		GOCmdContext *cc = go_component_get_command_context ();
+		GOCmdContext *cc = go_component_get_command_context (component);
 		GOIOContext *io_context = go_io_context_new (cc);
 		GsfOutput *output = gsf_output_memory_new ();
 		GOFileSaver *gfs = workbook_get_file_saver (gognm->wb);
@@ -100,6 +101,7 @@ go_gnm_component_get_data (GOComponent *component, gpointer *data, int *length,
 		*length = gsf_output_size (output);
 		*clearfunc = g_object_unref;
 		*user_data = output;
+		g_object_unref (io_context);
 		return TRUE;
 	}
 	return FALSE;
@@ -130,7 +132,7 @@ static void
 go_gnm_component_set_data (GOComponent *component)
 {
 	GOGnmComponent *gognm = GO_GNM_COMPONENT (component);
-	GOCmdContext *cc = go_component_get_command_context ();
+	GOCmdContext *cc = go_component_get_command_context (component);
 	GOIOContext *io_context = go_io_context_new (cc);
 	GsfInput *input = gsf_input_memory_new (component->data, component->length, FALSE);
 
@@ -139,8 +141,9 @@ go_gnm_component_set_data (GOComponent *component)
 		g_object_unref (gognm->wv);
 		g_object_unref (gognm->wb);
 	}
-	gognm->wv = wb_view_new_from_input (input, NULL, NULL, io_context, NULL);
+	gognm->wv = workbook_view_new_from_input (input, NULL, NULL, io_context, NULL);
 	gognm->wb = wb_view_get_workbook (gognm->wv);
+	gnm_app_workbook_list_remove (gognm->wb);
 	g_object_unref (io_context);
 	go_gnm_component_update_data (gognm);
 }
@@ -182,7 +185,8 @@ cb_gognm_save (G_GNUC_UNUSED GtkAction *a, WBCGtk *wbcg)
 				g_object_unref (gognm->wb);
 			}
 			gognm->wv = g_object_ref (wv);
-			gognm->wb = g_object_ref (wb_view_get_workbook (wv));
+			gognm->wb = wb_view_get_workbook (wv);
+			gnm_app_workbook_list_remove (gognm->wb); /* no need to have this one in the list */
 		}
 		go_doc_set_dirty (GO_DOC (gognm->wb), FALSE);
 		go_gnm_component_update_data (gognm);
@@ -201,6 +205,8 @@ static GtkActionEntry const actions[] = {
 static void
 cb_editor_destroyed (GOGnmComponent *gognm)
 {
+	if (gognm->edited && G_OBJECT (gognm->edited)->ref_count > 0)
+		g_object_unref (gognm->edited);
 	gognm->edited = NULL;
 }
 
@@ -219,12 +225,12 @@ go_gnm_component_edit (GOComponent *component)
 		component->width = 0.;
 		wv = workbook_view_new (workbook_new_with_sheets (1));
 	} else {
-		GOCmdContext *cc = go_component_get_command_context ();
+		GOCmdContext *cc = go_component_get_command_context (component);
 		GOIOContext *io_context = GO_IS_IO_CONTEXT (cc)? GO_IO_CONTEXT (g_object_ref (cc)): go_io_context_new (cc);
 		GsfInput *input = gsf_input_memory_new (component->data, component->length, FALSE);
 
 		g_object_set (G_OBJECT (io_context), "exec-main-loop", FALSE, NULL);
-		wv = wb_view_new_from_input (input, NULL, NULL, io_context, NULL);
+		wv = workbook_view_new_from_input (input, NULL, NULL, io_context, NULL);
 		g_object_unref (io_context);
 	}
 	set_uifilename ("Gnumeric-embed.xml", actions, G_N_ELEMENTS (actions));
@@ -318,7 +324,7 @@ go_plugin_init (GOPlugin *plugin, G_GNUC_UNUSED GOCmdContext *cc)
 
 	go_components_set_mime_suffix ("application/x-gnumeric", "*.gnumeric");
 
-	go_plugins_init (go_component_get_command_context (),
+	go_plugins_init (go_component_get_command_context (NULL),
 			gnm_conf_get_plugins_file_states (),
 			gnm_conf_get_plugins_active (),
 			dir_list,
