@@ -91,6 +91,11 @@ static GnmApp *app;
 
 static Workbook *gnm_app_workbook_get_by_uri (char const *uri);
 
+/**
+ * gnm_app_get_app:
+ *
+ * Returns: (transfer none): the #GnmApp instance.
+ **/
 GObject *
 gnm_app_get_app (void)
 {
@@ -98,8 +103,8 @@ gnm_app_get_app (void)
 }
 
 /**
- * gnm_app_workbook_list_add :
- * @wb :
+ * gnm_app_workbook_list_add:
+ * @wb:
  *
  * Add @wb to the application's list of workbooks.
  **/
@@ -118,8 +123,8 @@ gnm_app_workbook_list_add (Workbook *wb)
 }
 
 /**
- * gnm_app_workbook_list_remove :
- * @wb :
+ * gnm_app_workbook_list_remove:
+ * @wb:
  *
  * Remove @wb from the application's list of workbooks.
  **/
@@ -136,6 +141,11 @@ gnm_app_workbook_list_remove (Workbook *wb)
 	g_signal_emit (G_OBJECT (app), signals [WORKBOOK_REMOVED], 0, wb);
 }
 
+/**
+ * gnm_app_workbook_list:
+ *
+ * Returns: (element-type Workbook) (transfer none): the workbook list.
+ **/
 GList *
 gnm_app_workbook_list (void)
 {
@@ -201,11 +211,11 @@ gnm_app_clipboard_unant (void)
 /**
  * gnm_app_clipboard_cut_copy:
  *
- * @wbc   : the workbook control that requested the operation.
+ * @wbc: the workbook control that requested the operation.
  * @is_cut: is this a cut or a copy.
- * @sheet : The source sheet for the copy.
- * @area  : A single rectangular range to be copied.
- * @animate_cursor : Do we want ot add an animated cursor around things.
+ * @sv: The source sheet for the copy.
+ * @area: A single rectangular range to be copied.
+ * @animate_range: Do we want ot add an animated cursor around things.
  *
  * When Cutting we
  *   Clear and free the contents of the clipboard and save the sheet and area
@@ -253,11 +263,12 @@ gnm_app_clipboard_cut_copy (WorkbookControl *wbc, gboolean is_cut,
 	}
 }
 
-/** gnm_app_clipboard_cut_copy_obj :
- * @wbc : #WorkbookControl
- * @is_cut :
- * @sv : #SheetView
- * @objects : a list of #SheetObject which is freed
+/**
+ * gnm_app_clipboard_cut_copy_obj:
+ * @wbc: #WorkbookControl
+ * @is_cut:
+ * @sv: #SheetView
+ * @objects: (element-type SheetObject): a list of #SheetObject which is freed
  *
  * Different than copying/cutting a region, this can actually cuts an object
  **/
@@ -306,6 +317,11 @@ gnm_app_clipboard_is_cut (void)
 	return FALSE;
 }
 
+/**
+ * gnm_app_clipboard_sheet_get:
+ *
+ * Returns: (transfer none): the current clipboard #Sheet.
+ **/
 Sheet *
 gnm_app_clipboard_sheet_get (void)
 {
@@ -316,6 +332,11 @@ gnm_app_clipboard_sheet_get (void)
 	return sv_sheet (app->clipboard_sheet_view);
 }
 
+/**
+ * gnm_app_clipboard_sheet_view_get:
+ *
+ * Returns: (transfer none): the current clipboard #SheetView.
+ **/
 SheetView *
 gnm_app_clipboard_sheet_view_get (void)
 {
@@ -345,6 +366,13 @@ gnm_app_clipboard_area_get (void)
 	return NULL;
 }
 
+/**
+ * gnm_app_workbook_get_by_name:
+ * @name: the workbook name.
+ * @ref_uri:
+ *
+ * Returns: (transfer none): the #Workbook or %NULL.
+ **/
 Workbook *
 gnm_app_workbook_get_by_name (char const *name,
 			      char const *ref_uri)
@@ -444,7 +472,7 @@ gnm_app_workbook_foreach (GnmWbIterFunc cback, gpointer data)
 /**
  * gnm_app_workbook_get_by_index:
  * @i: index
- * 
+ *
  * Get nth workbook.  Index is zero-based.
  * Return value: (transfer none): the nth workbook if any.
  */
@@ -470,6 +498,14 @@ gnm_app_dpi_to_pixels (void)
 }
 
 /* GtkFileFilter */
+/**
+ * gnm_app_create_opener_filter:
+ * @openers: (element-type GOFileOpener): a list of file openers.
+ *
+ * Creates a #GtkFileFilter from the list of file types supported by the
+ * openers in the list.
+ * Returns: (transfer full): the newly allocated #GtkFileFilter.
+ **/
 void *
 gnm_app_create_opener_filter (GList *openers)
 {
@@ -559,7 +595,8 @@ compare_mru (GtkRecentInfo *a, GtkRecentInfo *b)
  *
  * creating it if necessary.
  *
- * Return value: the list, which must be freed along with the strings in it.
+ * Return value: (element-type char) (transfer full): the list, which must be
+ * freed along with the strings in it.
  **/
 GSList *
 gnm_app_history_get_list (int max_elements)
@@ -862,6 +899,16 @@ GSF_CLASS (GnmApp, gnm_app,
 /**********************************************************************/
 static GSList *extra_uis = NULL;
 
+/**
+ * gnm_action_new:
+ * @name: action ID.
+ * @label: label.
+ * @icon: icon name.
+ * @always_available: whether the action should always be available.
+ * @handler: (scope async): the handler.
+ *
+ * Returns: (transfer full): the newly allocated #GnmAction.
+ **/
 GnmAction *
 gnm_action_new (char const *id, char const *label,
 		char const *icon_name, gboolean always_available,
@@ -887,6 +934,59 @@ gnm_action_free (GnmAction *action)
 	}
 }
 
+static GnmAction *
+gnm_action_copy (GnmAction const *action)
+{
+	return gnm_action_new (action->id, action->label, action->icon_name,
+	                       action->always_available, action->handler);
+}
+
+GType
+gnm_action_get_type (void)
+{
+	static GType t = 0;
+
+	if (t == 0) {
+		t = g_boxed_type_register_static ("GnmAction",
+			 (GBoxedCopyFunc)gnm_action_copy,
+			 (GBoxedFreeFunc)gnm_action_free);
+	}
+	return t;
+}
+
+/***************
+ * making GnmAppExtraUI a boxed type for introspection. copy and free don't do
+ anything which might be critical, crossing fingers.*/
+
+static GnmAppExtraUI *
+gnm_app_extra_ui_ref (GnmAppExtraUI *ui)
+{
+	return ui;
+}
+
+GType
+gnm_app_extra_ui_get_type (void)
+{
+	static GType t = 0;
+
+	if (t == 0) {
+		t = g_boxed_type_register_static ("GnmAppExtraUI",
+			 (GBoxedCopyFunc)gnm_app_extra_ui_ref,
+			 (GBoxedFreeFunc)gnm_app_extra_ui_ref);
+	}
+	return t;
+}
+
+/**
+ * gnm_app_add_extra_ui:
+ * @group_name: action group name.
+ * @actions: (element-type GnmAction): list of actions.
+ * @layout: the xml string describing the menus and toolbars.
+ * @domain: localization domain.
+ * @user_data: user data
+ *
+ * Returns: (transfer full): the newly allocated #GnmAppExtraUI.
+ **/
 GnmAppExtraUI *
 gnm_app_add_extra_ui (char const *group_name,
 		      GSList *actions,
@@ -913,6 +1013,13 @@ gnm_app_remove_extra_ui (GnmAppExtraUI *extra_ui)
 	g_free (extra_ui);
 }
 
+/**
+ * gnm_app_foreach_extra_ui:
+ * @func: (scope call): #GFunc
+ * @data: user data.
+ *
+ * Applies @func to each #GnmAppExtraUI.
+ **/
 void
 gnm_app_foreach_extra_ui (GFunc func, gpointer data)
 {
@@ -932,7 +1039,7 @@ cb_flag_windows_changed (void)
 }
 
 /**
- * _gnm_app_flag_windows_changed :
+ * _gnm_app_flag_windows_changed:
  *
  * An internal utility routine to flag a regeneration of the window lists
  **/
@@ -946,6 +1053,31 @@ _gnm_app_flag_windows_changed (void)
 
 /**********************************************************************/
 
+/**
+ * gnm_app_recalc:
+ *
+ * Recalculate everything dirty in all workbooks that have automatic
+ * recalc turned on.
+ **/
+void
+gnm_app_recalc (void)
+{
+	GList *l;
+
+	g_return_if_fail (app != NULL);
+
+	gnm_app_recalc_start ();
+
+	for (l = app->workbook_list; l; l = l->next) {
+		Workbook *wb = l->data;
+
+		if (workbook_get_recalcmode (wb))
+			workbook_recalc (wb);
+	}
+
+	gnm_app_recalc_finish ();
+}
+
 void
 gnm_app_recalc_start (void)
 {
