@@ -36,6 +36,28 @@
 #include "rendered-value.h"
 #include <goffice/goffice.h>
 
+/* Making ColRowInfo a boxed type to make introspection happy. using no-op
+ * functions for copy and free, and  crossing fingers.
+ */
+static ColRowInfo *
+col_row_info_copy (ColRowInfo *cri)
+{
+	return cri;
+}
+
+GType
+col_row_info_get_type (void)
+{
+	static GType t = 0;
+
+	if (t == 0) {
+		t = g_boxed_type_register_static ("ColRowInfo",
+			 (GBoxedCopyFunc)col_row_info_copy,
+			 (GBoxedFreeFunc)col_row_info_copy);
+	}
+	return t;
+}
+
 double
 colrow_compute_pixel_scale (Sheet const *sheet, gboolean horizontal)
 {
@@ -143,7 +165,7 @@ colrow_copy (ColRowInfo *dst, ColRowInfo const *src)
 }
 
 ColRowInfo *
-colrow_new (void)
+col_row_info_new (void)
 {
 	return g_slice_new (ColRowInfo);
 }
@@ -156,13 +178,13 @@ colrow_free (ColRowInfo *cri)
 
 /**
  * colrow_foreach:
- * @sheet	the sheet
- * @infos	The Row or Column collection.
- * @start	start position (inclusive)
- * @end		stop column (inclusive)
- * @callback	A callback function which should return TRUE to stop
+ * @sheet:	the sheet
+ * @infos:	The Row or Column collection.
+ * @start:	start position (inclusive)
+ * @end	:	stop column (inclusive)
+ * @callback: (scope call): A callback function which should return TRUE to stop
  *              the iteration.
- * @user_data	A bagage pointer.
+ * @user_data:	A bagage pointer.
  *
  * Iterates through the existing rows or columns within the range supplied.
  * Currently only support left -> right iteration.  If a callback returns
@@ -225,6 +247,12 @@ colrow_vis_list_length (ColRowVisList *list)
 	return count;
 }
 
+/**
+ * colrow_state_group_destroy:
+ * @set: (transfer full): the group to destroy.
+ *
+ * Returns: (transfer none): %NULL.
+ **/
 ColRowStateGroup *
 colrow_state_group_destroy (ColRowStateGroup *group)
 {
@@ -284,10 +312,15 @@ colrow_index_list_to_string (ColRowIndexList *list, gboolean is_cols, gboolean *
 }
 
 /**
- * colrow_get_index_list :
+ * colrow_get_index_list:
+ * @first:
+ * @last:
+ * @list: (transfer full):
  *
  * Build an ordered list of pairs doing intelligent merging
  * of overlapping regions.
+ *
+ * Returns: (transfer full): @list.
  */
 ColRowIndexList *
 colrow_get_index_list (int first, int last, ColRowIndexList *list)
@@ -321,6 +354,12 @@ colrow_get_index_list (int first, int last, ColRowIndexList *list)
 	return list;
 }
 
+/**
+ * colrow_index_list_copy:
+ * @list: #ColRowIndexList
+ *
+ * Returns: (transfer full):
+ **/
 ColRowIndexList *
 colrow_index_list_copy (ColRowIndexList *list)
 {
@@ -349,6 +388,12 @@ colrow_set_single_state (ColRowState *state,
 	state->visible = info->visible;
 }
 
+/**
+ * colrow_state_list_destroy:
+ * @list: (transfer full): the list to destroy.
+ *
+ * Returns: (transfer none): %NULL.
+ **/
 ColRowStateList *
 colrow_state_list_destroy (ColRowStateList *list)
 {
@@ -356,6 +401,15 @@ colrow_state_list_destroy (ColRowStateList *list)
 	return NULL;
 }
 
+/**
+ * colrow_get_states:
+ * @sheet: #Sheet
+ * @is_cols: %TRUE if columns.
+ * @first:
+ * @last:
+ *
+ * Returns: (transfer full):
+ **/
 ColRowStateList *
 colrow_get_states (Sheet *sheet, gboolean is_cols, int first, int last)
 {
@@ -435,6 +489,15 @@ cb_clear_variable_width_content (GnmCellIter const *iter,
 	return NULL;
 }
 
+/**
+ * colrow_get_sizes:
+ * @sheet: #Sheet
+ * @is_cols: %TRUE if columns.
+ * @src:
+ * @new_size:
+ *
+ * Returns: (transfer full):
+ **/
 ColRowStateGroup *
 colrow_get_sizes (Sheet *sheet, gboolean is_cols,
 		  ColRowIndexList *src, int new_size)
@@ -466,6 +529,17 @@ colrow_get_sizes (Sheet *sheet, gboolean is_cols,
 	return res;
 }
 
+/**
+ * colrow_set_sizes:
+ * @sheet: #Sheet
+ * @is_cols:
+ * @src:
+ * @new_size:
+ * @from:
+ * @to:
+ *
+ * Returns: (transfer full):
+ **/
 ColRowStateGroup *
 colrow_set_sizes (Sheet *sheet, gboolean is_cols,
 		  ColRowIndexList *src, int new_size, int from, int to)
@@ -911,6 +985,16 @@ colrow_visibility (Sheet const *sheet, ColRowVisiblity * const dat,
 	}
 }
 
+/**
+ * colrow_get_outline_toggle:
+ * @sheet: #Sheet
+ * @is_cols:
+ * @visible:
+ * @first:
+ * @last:
+ *
+ * Returns: (transfer full):
+ **/
 ColRowVisList *
 colrow_get_outline_toggle (Sheet const *sheet, gboolean is_cols, gboolean visible,
 			   int first, int last)
@@ -941,16 +1025,18 @@ cb_colrow_visibility (SheetView *sv, GnmRange const *r, gpointer closure)
 }
 
 /**
- * colrow_get_visiblity_toggle :
- * @sheet : The sheet whose selection we are interested in.
+ * colrow_get_visiblity_toggle:
+ * @sv: The sheet view whose selection we are interested in.
  * @is_cols: A flag indicating whether this it is a column or a row.
- * @is_visible: Should we unhide or hide the cols/rows.
+ * @visible: Should we unhide or hide the cols/rows.
  *
  * Searches the selection list and generates a list of index,count
  * pairs of row/col ranges that need to be hidden or unhiden.
  *
  * NOTE : leave sheet non-const until we have a const version of
  *        sv_selection_apply.
+ *
+ * Returns: (transfer full): the list.
  */
 ColRowVisList *
 colrow_get_visiblity_toggle (SheetView *sv, gboolean is_cols,
@@ -1043,11 +1129,11 @@ colrow_find_outline_bound (Sheet const *sheet, gboolean is_cols,
 
 /**
  * colrow_set_visibility:
- * @sheet	: the sheet
- * @is_cols	: Are we dealing with rows or columns.
- * @visible	: Make things visible or invisible.
- * @first	: The index of the first row/col (inclusive)
- * @last	: The index of the last row/col (inclusive)
+ * @sheet: the sheet
+ * @is_cols: Are we dealing with rows or columns.
+ * @visible: Make things visible or invisible.
+ * @first: The index of the first row/col (inclusive)
+ * @last: The index of the last row/col (inclusive)
  *
  * Change the visibility of the selected range of contiguous cols/rows.
  * NOTE : only changes the collapsed state for the LAST+1 element.
