@@ -3,7 +3,7 @@
  * dependent.c:  Manage calculation dependencies between objects
  *
  * Copyright (C) 2000-2006 Jody Goldberg (jody@gnome.org)
- * Copyright (C) 2006-2009 Morten Welinder (terra@gnome.org)
+ * Copyright (C) 2006-2012 Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,7 +33,6 @@
 #include "expr-name.h"
 #include "application.h"
 #include "workbook-view.h"
-#include "rendered-value.h" /* FIXME : should not be needed with JIT-R */
 #include "ranges.h"
 #include "gutils.h"
 #include "sheet-view.h"
@@ -42,6 +41,7 @@
 #include <goffice/goffice.h>
 #include <string.h>
 
+static gboolean gnm_cell_eval_content (GnmCell *cell);
 static void dependent_changed (GnmDependent *dep);
 static void dependent_clear_dynamic_deps (GnmDependent *dep);
 
@@ -231,28 +231,41 @@ gnm_dep_unlink_undo_new (GSList *deps)
 
 #undef DEBUG_EVALUATION
 
+static void cell_dep_eval	   (GnmDependent *dep);
+static void cell_dep_set_expr	   (GnmDependent *dep, GnmExprTop const *new_texpr);
+static GSList *cell_dep_changed	   (GnmDependent *dep);
+static void cell_dep_debug_name    (GnmDependent const *dep, GString *target);
 static void dynamic_dep_eval	   (GnmDependent *dep);
+static GSList *dynamic_dep_changed (GnmDependent *dep);
 static void dynamic_dep_debug_name (GnmDependent const *dep, GString *target);
 static void name_dep_eval	   (GnmDependent *dep);
 static void name_dep_debug_name	   (GnmDependent const *dep, GString *target);
 static void managed_dep_eval	   (GnmDependent *dep);
 static void managed_dep_debug_name (GnmDependent const *dep, GString *target);
 
-static GnmCellPos const dummy = { 0, 0 };
 static GPtrArray *dep_classes = NULL;
+static GnmDependentClass cell_dep_class = {
+	cell_dep_eval,
+	cell_dep_set_expr,
+	cell_dep_changed,
+	cell_dep_debug_name,
+};
 static GnmDependentClass dynamic_dep_class = {
 	dynamic_dep_eval,
 	NULL,
+	dynamic_dep_changed,
 	dynamic_dep_debug_name,
 };
 static GnmDependentClass name_dep_class = {
 	name_dep_eval,
 	NULL,
+	NULL,
 	name_dep_debug_name,
 };
 static GnmDependentClass managed_dep_class = {
 	managed_dep_eval,
 	NULL,
+	NULL,
 	managed_dep_debug_name,
 };
 typedef struct {
@@ -270,7 +283,7 @@ dependent_types_init (void)
 	/* Init with a trio of NULL classes so we can access directly */
 	dep_classes = g_ptr_array_new ();
 	g_ptr_array_add	(dep_classes, NULL); /* bogus filler */
-	g_ptr_array_add	(dep_classes, NULL); /* Cell */
+	g_ptr_array_add	(dep_classes, &cell_dep_class);
 	g_ptr_array_add	(dep_classes, &dynamic_dep_class);
 	g_ptr_array_add	(dep_classes, &name_dep_class);
 	g_ptr_array_add	(dep_classes, &managed_dep_class);
@@ -365,27 +378,18 @@ void
 dependent_set_expr (GnmDependent *dep, GnmExprTop const *new_texpr)
 {
 	int const t = dependent_type (dep);
+	GnmDependentClass *klass = g_ptr_array_index (dep_classes, t);
 
 	if (dependent_is_linked (dep))
 		dependent_unlink (dep);
 	if (dep->flags & DEPENDENT_HAS_DYNAMIC_DEPS)
 		dependent_clear_dynamic_deps (dep);
 
-	if (t == DEPENDENT_CELL) {
-		/*
-		 * Explicitly do not check for array subdivision, we may be
-		 * replacing the corner of an array.
-		 */
-		gnm_cell_set_expr_unsafe (GNM_DEP_TO_CELL (dep), new_texpr);
-	} else {
-		GnmDependentClass *klass = g_ptr_array_index (dep_classes, t);
-
-		g_return_if_fail (klass);
+	if (klass->set_expr)
+		klass->set_expr (dep, new_texpr);
+	else {
 		if (new_texpr)
 			gnm_expr_top_ref (new_texpr);
-		if (klass->set_expr)
-			klass->set_expr (dep, new_texpr);
-
 		if (dep->texpr)
 			gnm_expr_top_unref (dep->texpr);
 		dep->texpr = new_texpr;
@@ -397,6 +401,7 @@ dependent_set_expr (GnmDependent *dep, GnmExprTop const *new_texpr)
 static inline GnmCellPos const *
 dependent_pos (GnmDependent const *dep)
 {
+	static GnmCellPos const dummy = { 0, 0 };
 	return dependent_is_cell (dep) ? &GNM_DEP_TO_CELL (dep)->pos : &dummy;
 }
 
@@ -448,34 +453,16 @@ dependent_queue_recalc_main (GSList *work)
 	while (work) {
 		GnmDependent *dep = work->data;
 		int const t = dependent_type (dep);
+		GnmDependentClass *klass = g_ptr_array_index (dep_classes, t);
+
 		/* Pop the top element.  */
-		GSList *list = work;
-		work = work->next;
+		work = g_slist_delete_link (work, work);
 
-		g_slist_free_1 (list);
-
-		if (t == DEPENDENT_CELL) {
-			GSList *deps = cell_list_deps (GNM_DEP_TO_CELL (dep));
-			GSList *waste = NULL;
-			GSList *next;
-			for (list = deps; list != NULL ; list = next) {
-				GnmDependent *dep = list->data;
-				next = list->next;
-				if (dependent_needs_recalc (dep)) {
-					list->next = waste;
-					waste = list;
-				} else {
-					dependent_flag_recalc (dep);
-					list->next = work;
-					work = list;
-				}
-			}
-			g_slist_free (waste);
-		} else if (t == DEPENDENT_DYNAMIC_DEP) {
-			DynamicDep const *dyn = (DynamicDep *)dep;
-			if (!dependent_needs_recalc (dyn->container)) {
-				dependent_flag_recalc (dyn->container);
-				work = g_slist_prepend (work, dyn->container);
+		if (klass->changed) {
+			GSList *extra = klass->changed (dep);
+			if (extra) {
+				g_slist_last (extra)->next = work;
+				work = extra;
 			}
 		}
 	}
@@ -841,22 +828,20 @@ depsingle_equal (DependencySingle const *a, DependencySingle const *b)
 	return (a->pos.row == b->pos.row && a->pos.col == b->pos.col);
 }
 
-static DependentFlags
+static GnmDependentFlags
 link_single_dep (GnmDependent *dep, GnmCellPos const *pos, GnmCellRef const *ref)
 {
 	DependencySingle lookup;
 	DependencySingle *single;
-	GnmDepContainer *deps;
-	DependentFlags flag = DEPENDENT_NO_FLAG;
+	GnmDependentFlags flag = DEPENDENT_NO_FLAG;
 	Sheet const *sheet = eval_sheet (ref->sheet, dep->sheet);
+	GnmDepContainer *deps = sheet->deps;
 
 	if (sheet != dep->sheet)
 		flag = (sheet->workbook != dep->sheet->workbook)
 			? DEPENDENT_GOES_INTERBOOK
 			: DEPENDENT_GOES_INTERSHEET;
 
-	deps = sheet->deps;
-
 	/* Inserts if it is not already there */
 	gnm_cellpos_init_cellref (&lookup.pos, ref, pos, sheet);
 	single = g_hash_table_lookup (deps->single_hash, &lookup);
@@ -871,16 +856,21 @@ link_single_dep (GnmDependent *dep, GnmCellPos const *pos, GnmCellRef const *ref
 	return flag;
 }
 
-static void
+static GnmDependentFlags
 unlink_single_dep (GnmDependent *dep, GnmCellPos const *pos, GnmCellRef const *a)
 {
 	DependencySingle lookup;
 	DependencySingle *single;
+	GnmDependentFlags flag = DEPENDENT_NO_FLAG;
 	Sheet const *sheet = eval_sheet (a->sheet, dep->sheet);
 	GnmDepContainer *deps = sheet->deps;
 
+	if (sheet != dep->sheet)
+		flag = (sheet->workbook != dep->sheet->workbook)
+			? DEPENDENT_GOES_INTERBOOK
+			: DEPENDENT_GOES_INTERSHEET;
 	if (!deps)
-		return;
+		return flag;
 
 	gnm_cellpos_init_cellref (&lookup.pos, a, pos, sheet);
 	single = g_hash_table_lookup (deps->single_hash, &lookup);
@@ -892,8 +882,20 @@ unlink_single_dep (GnmDependent *dep, GnmCellPos const *pos, GnmCellRef const *a
 			go_mem_chunk_free (deps->single_pool, single);
 		}
 	}
+
+	return flag;
 }
 
+static inline GnmDependentFlags
+link_unlink_single_dep (GnmDependent *dep, GnmCellPos const *pos,
+			GnmCellRef const *a, gboolean qlink)
+{
+	return qlink
+		? link_single_dep (dep, pos, a)
+		: unlink_single_dep (dep, pos, a);
+}
+
+
 static void
 link_range_dep (GnmDepContainer *deps, GnmDependent *dep,
 		DependencyRange const *r)
@@ -960,12 +962,23 @@ unlink_range_dep (GnmDepContainer *deps, GnmDependent *dep,
 	}
 }
 
-static DependentFlags
-link_cellrange_dep (GnmDependent *dep, GnmCellPos const *pos,
-		    GnmCellRef const *a, GnmCellRef const *b)
+static inline void
+link_unlink_range_dep (GnmDepContainer *deps, GnmDependent *dep,
+		       DependencyRange const *r, gboolean qlink)
+{
+	if (qlink)
+		link_range_dep (deps, dep, r);
+	else
+		unlink_range_dep (deps, dep, r);
+}
+
+static GnmDependentFlags
+link_unlink_cellrange_dep (GnmDependent *dep, GnmCellPos const *pos,
+			   GnmCellRef const *a, GnmCellRef const *b,
+			   gboolean qlink)
 {
 	DependencyRange range;
-	DependentFlags flag = DEPENDENT_NO_FLAG;
+	GnmDependentFlags flag = DEPENDENT_NO_FLAG;
 
 	gnm_cellpos_init_cellref (&range.range.start, a, pos, dep->sheet);
 	gnm_cellpos_init_cellref (&range.range.end, b, pos, dep->sheet);
@@ -987,49 +1000,21 @@ link_cellrange_dep (GnmDependent *dep, GnmCellPos const *pos,
 			while (i <= stop) {
 				Sheet *sheet = g_ptr_array_index (wb->sheets, i);
 				i++;
-				link_range_dep (sheet->deps, dep, &range);
+				link_unlink_range_dep (sheet->deps, dep, &range,
+						       qlink);
 			}
 			flag |= DEPENDENT_HAS_3D;
 		} else
-			link_range_dep (a->sheet->deps, dep, &range);
+			link_unlink_range_dep (a->sheet->deps, dep, &range,
+					       qlink);
 	} else
-		link_range_dep (dep->sheet->deps, dep, &range);
+		link_unlink_range_dep (dep->sheet->deps, dep, &range, qlink);
 
 	return flag;
 }
 
-static void
-unlink_cellrange_dep (GnmDependent *dep, GnmCellPos const *pos,
-		      GnmCellRef const *a, GnmCellRef const *b)
-{
-	DependencyRange range;
-
-	gnm_cellpos_init_cellref (&range.range.start, a, pos, dep->sheet);
-	gnm_cellpos_init_cellref (&range.range.end, b, pos, dep->sheet);
-	range_normalize (&range.range);
-
-	if (a->sheet != NULL) {
-		if (b->sheet != NULL && a->sheet != b->sheet) {
-			Workbook const *wb = a->sheet->workbook;
-			int i = a->sheet->index_in_wb;
-			int stop = b->sheet->index_in_wb;
-			if (i > stop) { int tmp = i; i = stop ; stop = tmp; }
-
-			g_return_if_fail (b->sheet->workbook == wb);
-
-			while (i <= stop) {
-				Sheet *sheet = g_ptr_array_index (wb->sheets, i);
-				i++;
-				unlink_range_dep (sheet->deps, dep, &range);
-			}
-		} else
-			unlink_range_dep (a->sheet->deps, dep, &range);
-	} else
-		unlink_range_dep (dep->sheet->deps, dep, &range);
-}
-
-static DependentFlags
-link_expr_dep (GnmEvalPos *ep, GnmExpr const *tree)
+static GnmDependentFlags
+link_unlink_expr_dep (GnmEvalPos *ep, GnmExpr const *tree, gboolean qlink)
 {
 	g_return_val_if_fail (tree != NULL, DEPENDENT_NO_FLAG);
 
@@ -1037,45 +1022,53 @@ link_expr_dep (GnmEvalPos *ep, GnmExpr const *tree)
 	case GNM_EXPR_OP_RANGE_CTOR:  /* See #562363 */
 	case GNM_EXPR_OP_INTERSECT:
 	case GNM_EXPR_OP_ANY_BINARY:
-		return link_expr_dep (ep, tree->binary.value_a) |
-		       link_expr_dep (ep, tree->binary.value_b);
+		return link_unlink_expr_dep (ep, tree->binary.value_a, qlink) |
+		       link_unlink_expr_dep (ep, tree->binary.value_b, qlink);
 	case GNM_EXPR_OP_ANY_UNARY:
-		return link_expr_dep (ep, tree->unary.value);
+		return link_unlink_expr_dep (ep, tree->unary.value, qlink);
 	case GNM_EXPR_OP_CELLREF:
-		return link_single_dep (ep->dep, dependent_pos (ep->dep), &tree->cellref.ref);
+		return link_unlink_single_dep (ep->dep, dependent_pos (ep->dep), &tree->cellref.ref, qlink);
 
 	case GNM_EXPR_OP_CONSTANT:
 		/* TODO: pass in eval flags so that we can use implicit
 		 * intersection
 		 */
 		if (VALUE_CELLRANGE == tree->constant.value->type)
-			return link_cellrange_dep (ep->dep, dependent_pos (ep->dep),
+			return link_unlink_cellrange_dep
+				(ep->dep, dependent_pos (ep->dep),
 				 &tree->constant.value->v_range.cell.a,
-				 &tree->constant.value->v_range.cell.b);
+				 &tree->constant.value->v_range.cell.b,
+				 qlink);
 		return DEPENDENT_NO_FLAG;
 
-	/* TODO : Can we use argument types to be smarter here ? */
+	/*
+	 * FIXME: needs to be taught implicit intersection +
+	 * more cunning handling of argument type matching.
+	 */
 	case GNM_EXPR_OP_FUNCALL: {
 		int i;
-		DependentFlags flag = DEPENDENT_NO_FLAG;
+		GnmDependentFlags flag = DEPENDENT_NO_FLAG;
 		if (tree->func.func->fn_type == GNM_FUNC_TYPE_STUB)
 			gnm_func_load_stub (tree->func.func);
 		if (tree->func.func->linker) {
 			GnmFuncEvalInfo fei;
 			fei.pos = ep;
 			fei.func_call = &tree->func;
-			flag = tree->func.func->linker (&fei);
+			flag = tree->func.func->linker (&fei, qlink);
 		}
 		if (!(flag & DEPENDENT_IGNORE_ARGS))
 			for (i = 0; i < tree->func.argc; i++)
-				flag |= link_expr_dep (ep, tree->func.argv[i]);
+				flag |= link_unlink_expr_dep (ep, tree->func.argv[i], qlink);
 		return flag;
 	}
 
 	case GNM_EXPR_OP_NAME:
-		expr_name_add_dep (tree->name.name, ep->dep);
+		if (qlink)
+			expr_name_add_dep (tree->name.name, ep->dep);
+		else
+			expr_name_remove_dep (tree->name.name, ep->dep);
 		if (expr_name_is_active (tree->name.name))
-			return link_expr_dep (ep, tree->name.name->texpr->expr) | DEPENDENT_USES_NAME;
+			return link_unlink_expr_dep (ep, tree->name.name->texpr->expr, qlink) | DEPENDENT_USES_NAME;
 		return DEPENDENT_USES_NAME;
 
 	case GNM_EXPR_OP_ARRAY_ELEM: {
@@ -1092,22 +1085,22 @@ link_expr_dep (GnmEvalPos *ep, GnmExpr const *tree)
 		a.col   = pos->col - tree->array_elem.x;
 		a.row   = pos->row - tree->array_elem.y;
 
-		return link_single_dep (ep->dep, pos, &a);
+		return link_unlink_single_dep (ep->dep, pos, &a, qlink);
 	}
 
 	case GNM_EXPR_OP_ARRAY_CORNER: {
 		GnmEvalPos pos = *ep;
 		pos.array = &tree->array_corner;
 		/* Corner cell depends on the contents of the expr */
-		return link_expr_dep (&pos, tree->array_corner.expr);
+		return link_unlink_expr_dep (&pos, tree->array_corner.expr, qlink);
 	}
 
 	case GNM_EXPR_OP_SET: {
 		int i;
-		DependentFlags res = DEPENDENT_NO_FLAG;
+		GnmDependentFlags res = DEPENDENT_NO_FLAG;
 
 		for (i = 0; i < tree->set.argc; i++)
-			res |= link_expr_dep (ep, tree->set.argv[i]);
+			res |= link_unlink_expr_dep (ep, tree->set.argv[i], qlink);
 		return res;
 	}
 #ifndef DEBUG_SWITCH_ENUM
@@ -1115,93 +1108,7 @@ link_expr_dep (GnmEvalPos *ep, GnmExpr const *tree)
 		g_assert_not_reached ();
 #endif
 	}
-	return 0;
-}
-
-static void
-unlink_expr_dep (GnmDependent *dep, GnmExpr const *tree)
-{
-	switch (GNM_EXPR_GET_OPER (tree)) {
-	case GNM_EXPR_OP_RANGE_CTOR:  /* See #562363 */
-	case GNM_EXPR_OP_INTERSECT:
-	case GNM_EXPR_OP_ANY_BINARY:
-		unlink_expr_dep (dep, tree->binary.value_a);
-		unlink_expr_dep (dep, tree->binary.value_b);
-		return;
-
-	case GNM_EXPR_OP_ANY_UNARY:
-		unlink_expr_dep (dep, tree->unary.value);
-		return;
-	case GNM_EXPR_OP_CELLREF:
-		unlink_single_dep (dep, dependent_pos (dep), &tree->cellref.ref);
-		return;
-
-	case GNM_EXPR_OP_CONSTANT:
-		if (VALUE_CELLRANGE == tree->constant.value->type)
-			unlink_cellrange_dep (dep, dependent_pos (dep),
-				&tree->constant.value->v_range.cell.a,
-				&tree->constant.value->v_range.cell.b);
-		return;
-
-	/*
-	 * FIXME: needs to be taught implicit intersection +
-	 * more cunning handling of argument type matching.
-	 */
-	case GNM_EXPR_OP_FUNCALL: {
-		int i;
-		if (tree->func.func->unlinker) {
-			GnmEvalPos ep;
-			GnmFuncEvalInfo fei;
-			fei.pos = eval_pos_init_dep (&ep, dep);
-			fei.func_call = &tree->func;
-			tree->func.func->unlinker (&fei);
-		}
-		for (i = 0; i < tree->func.argc; i++)
-			unlink_expr_dep (dep, tree->func.argv[i]);
-		return;
-	}
-
-	case GNM_EXPR_OP_NAME:
-		expr_name_remove_dep (tree->name.name, dep);
-		if (expr_name_is_active (tree->name.name))
-			unlink_expr_dep (dep, tree->name.name->texpr->expr);
-		return;
-
-	case GNM_EXPR_OP_ARRAY_ELEM: {
-		GnmCellRef a;
-		GnmCellPos const *pos = dependent_pos (dep);
-
-		g_return_if_fail (pos != NULL);
-
-		/* Non-corner cells depend on the corner */
-		a.col_relative = a.row_relative = FALSE;
-		a.sheet = dep->sheet;
-		a.col   = pos->col - tree->array_elem.x;
-		a.row   = pos->row - tree->array_elem.y;
-
-		unlink_single_dep (dep, pos, &a);
-		return;
-	}
-
-	case GNM_EXPR_OP_ARRAY_CORNER:
-		/* Corner depends on the contents of the expr */
-		unlink_expr_dep (dep, tree->array_corner.expr);
-		return;
-
-	case GNM_EXPR_OP_SET: {
-		int i;
-
-		for (i = 0; i < tree->set.argc; i++)
-			unlink_expr_dep (dep, tree->set.argv[i]);
-		return;
-	}
-
-#ifndef DEBUG_SWITCH_ENUM
-	default:
-		g_assert_not_reached ();
-		break;
-#endif
-	}
+	return DEPENDENT_NO_FLAG;
 }
 
 static void
@@ -1232,6 +1139,56 @@ workbook_unlink_3d_dep (GnmDependent *dep)
 	g_hash_table_remove (wb->sheet_order_dependents, dep);
 }
 
+/*****************************************************************************/
+
+static void
+cell_dep_eval (GnmDependent *dep)
+{
+	gboolean finished = gnm_cell_eval_content (GNM_DEP_TO_CELL (dep));
+	(void)finished; /* We don't currently care */
+	dep->flags &= ~GNM_CELL_HAS_NEW_EXPR;
+}
+
+static void
+cell_dep_set_expr (GnmDependent *dep, GnmExprTop const *new_texpr)
+{
+	/*
+	 * Explicitly do not check for array subdivision, we may be
+	 * replacing the corner of an array.
+	 */
+	gnm_cell_set_expr_unsafe (GNM_DEP_TO_CELL (dep), new_texpr);
+}
+
+static GSList *
+cell_dep_changed (GnmDependent *dep)
+{
+	/* When a cell changes, so do its dependents.  */
+
+	GSList *deps = cell_list_deps (GNM_DEP_TO_CELL (dep));
+	GSList *waste = NULL, *work = NULL;
+	GSList *next, *list;
+	for (list = deps; list != NULL ; list = next) {
+		GnmDependent *dep = list->data;
+		next = list->next;
+		if (dependent_needs_recalc (dep)) {
+			list->next = waste;
+			waste = list;
+		} else {
+			dependent_flag_recalc (dep);
+			list->next = work;
+			work = list;
+		}
+	}
+	g_slist_free (waste);
+	return work;
+}
+
+static void
+cell_dep_debug_name (GnmDependent const *dep, GString *target)
+{
+	g_string_append (target, cell_name (GNM_DEP_TO_CELL (dep)));
+}
+
 /*****************************************************************************/
 /*
  * "Managed" dependent handling.
@@ -1255,7 +1212,10 @@ void
 dependent_managed_set_expr (GnmDependent *dep, GnmExprTop const *texpr)
 {
 	g_return_if_fail (dep != NULL);
+#if 0
+	/* We need some kind of IS_A */
 	g_return_if_fail (dependent_type (dep) == DEPENDENT_MANAGED);
+#endif
 
 	dependent_set_expr (dep, texpr);
 	if (texpr && dep->sheet)
@@ -1268,7 +1228,10 @@ dependent_managed_set_sheet (GnmDependent *dep, Sheet *sheet)
 	GnmExprTop const *texpr;
 
 	g_return_if_fail (dep != NULL);
+#if 0
+	/* We need some kind of IS_A */
 	g_return_if_fail (dependent_type (dep) == DEPENDENT_MANAGED);
+#endif
 
 	if (dep->sheet == sheet)
 		return;
@@ -1313,6 +1276,20 @@ dynamic_dep_eval (G_GNUC_UNUSED GnmDependent *dep)
 {
 }
 
+static GSList *
+dynamic_dep_changed (GnmDependent *dep)
+{
+	DynamicDep const *dyn = (DynamicDep *)dep;
+
+	/* When a dynamic dependent changes, we mark its container.  */
+
+	if (dependent_needs_recalc (dyn->container))
+		return NULL;
+
+	dependent_flag_recalc (dyn->container);
+	return g_slist_prepend (NULL, dyn->container);
+}
+
 static void
 dynamic_dep_debug_name (GnmDependent const *dep, GString *target)
 {
@@ -1322,7 +1299,7 @@ dynamic_dep_debug_name (GnmDependent const *dep, GString *target)
 void
 dependent_add_dynamic_dep (GnmDependent *dep, GnmRangeRef const *rr)
 {
-	DependentFlags    flags;
+	GnmDependentFlags    flags;
 	DynamicDep	 *dyn;
 	GnmCellPos const *pos;
 	DependencyRange   range;
@@ -1351,7 +1328,7 @@ dependent_add_dynamic_dep (GnmDependent *dep, GnmRangeRef const *rr)
 		flags = link_single_dep (&dyn->base, pos, &rr->a);
 		dyn->singles = g_slist_prepend (dyn->singles, gnm_rangeref_dup (rr));
 	} else {
-		flags = link_cellrange_dep (&dyn->base, pos, &rr->a, &rr->b);
+		flags = link_unlink_cellrange_dep (&dyn->base, pos, &rr->a, &rr->b, TRUE);
 		dyn->ranges = g_slist_prepend (dyn->ranges, gnm_rangeref_dup (rr));
 	}
 	if (flags & DEPENDENT_HAS_3D)
@@ -1368,7 +1345,7 @@ dependent_clear_dynamic_deps (GnmDependent *dep)
 
 /**
  * dependent_link:
- * @dep : the dependent that changed
+ * @dep: the dependent that changed
  *
  * Adds the dependent to the workbook wide list of dependents.
  */
@@ -1395,7 +1372,8 @@ dependent_link (GnmDependent *dep)
 		sheet->deps->head = dep; /* first element */
 	sheet->deps->tail = dep;
 	dep->flags |= DEPENDENT_IS_LINKED |
-		link_expr_dep (eval_pos_init_dep (&ep, dep), dep->texpr->expr);
+		link_unlink_expr_dep (eval_pos_init_dep (&ep, dep),
+				      dep->texpr->expr, TRUE);
 
 	if (dep->flags & DEPENDENT_HAS_3D)
 		workbook_link_3d_dep (dep);
@@ -1403,7 +1381,7 @@ dependent_link (GnmDependent *dep)
 
 /**
  * dependent_unlink:
- * @dep : the dependent that changed
+ * @dep: the dependent that changed
  *
  * Removes the dependent from its container's set of dependents and always
  * removes the linkages to what it depends on.
@@ -1412,13 +1390,15 @@ void
 dependent_unlink (GnmDependent *dep)
 {
 	GnmDepContainer *contain;
+	GnmEvalPos ep;
 
 	g_return_if_fail (dep != NULL);
 	g_return_if_fail (dependent_is_linked (dep));
 	g_return_if_fail (dep->texpr != NULL);
 	g_return_if_fail (IS_SHEET (dep->sheet));
 
-	unlink_expr_dep (dep, dep->texpr->expr);
+	link_unlink_expr_dep (eval_pos_init_dep (&ep, dep),
+			      dep->texpr->expr, FALSE);
 	contain = dep->sheet->deps;
 	if (contain != NULL) {
 		if (contain->head == dep)
@@ -1447,7 +1427,7 @@ dependent_unlink (GnmDependent *dep)
  * it should not be used by anyone. It is an internal
  * function.
  **/
-gboolean
+static gboolean
 gnm_cell_eval_content (GnmCell *cell)
 {
 	static GnmCell *iterating = NULL;
@@ -1459,15 +1439,6 @@ gnm_cell_eval_content (GnmCell *cell)
 	    !dependent_is_linked (&cell->base)) /* special case within TABLE */
 		return TRUE;
 
-	/* do this here rather than dependent_eval
-	 * because this routine is sometimes called
-	 * directly
-	 */
-	if (cell->base.flags & DEPENDENT_HAS_DYNAMIC_DEPS) {
-		dependent_clear_dynamic_deps (GNM_CELL_TO_DEP (cell));
-		cell->base.flags &= ~DEPENDENT_HAS_DYNAMIC_DEPS;
-	}
-
 #ifdef DEBUG_EVALUATION
 	{
 		GnmParsePos pp;
@@ -1521,6 +1492,11 @@ gnm_cell_eval_content (GnmCell *cell)
 	/* Prepare to calculate */
 	eval_pos_init_cell (&pos, cell);
 	cell->base.flags |= DEPENDENT_BEING_CALCULATED;
+
+	/*
+	 * I'm somewhat afraid of thinking about the semantics of iteration
+	 * if different workbooks have different settings.
+	 */
 	max_iteration = cell->base.sheet->workbook->iteration.max_number;
 
 iterate :
@@ -1604,34 +1580,34 @@ static void
 dependent_eval (GnmDependent *dep)
 {
 	int const t = dependent_type (dep);
+	GnmDependentClass *klass = g_ptr_array_index (dep_classes, t);
 
-	if (t != DEPENDENT_CELL) {
-		GnmDependentClass *klass = g_ptr_array_index (dep_classes, t);
-
-		g_return_if_fail (klass);
-
-		if (dep->flags & DEPENDENT_HAS_DYNAMIC_DEPS) {
-			dependent_clear_dynamic_deps (dep);
-			dep->flags &= ~DEPENDENT_HAS_DYNAMIC_DEPS;
-		}
-
-		klass->eval (dep);
-	} else {
-		/* This will clear the dynamic deps too, see comment there
-		 * to explain asymmetry.
-		 */
-		gboolean finished = gnm_cell_eval_content (GNM_DEP_TO_CELL (dep));
-
-		/* This should always be the top of the stack */
-		g_return_if_fail (finished);
-
-		dep->flags &= ~GNM_CELL_HAS_NEW_EXPR;
+	if (dep->flags & DEPENDENT_HAS_DYNAMIC_DEPS) {
+		dependent_clear_dynamic_deps (dep);
+		dep->flags &= ~DEPENDENT_HAS_DYNAMIC_DEPS;
 	}
 
+	/*
+	 * Problem: this really should be a tail call.
+	 */
+	klass->eval (dep);
+
 	/* Don't clear flag until after in case we iterate */
 	dep->flags &= ~DEPENDENT_NEEDS_RECALC;
 }
 
+void
+gnm_cell_eval (GnmCell *cell)
+{
+	if (gnm_cell_needs_recalc (cell)) {
+		/*
+		 * This should be a tail call so the stack frame can be
+		 * eliminated before the call.
+		 */
+		dependent_eval (GNM_CELL_TO_DEP (cell));
+	}
+}
+
 
 /**
  * cell_queue_recalc :
@@ -1658,8 +1634,8 @@ cell_queue_recalc (GnmCell *cell)
 }
 
 typedef struct {
-	int      col, row;
-	DepFunc	 func;
+	int col, row;
+	GnmDepFunc func;
 	gpointer user;
 } search_rangedeps_closure_t;
 
@@ -1679,14 +1655,14 @@ cb_search_rangedeps (gpointer key, G_GNUC_UNUSED gpointer value,
 #endif
 
 	if (range_contains (range, c->col, c->row)) {
-		DepFunc	 func = c->func;
+		GnmDepFunc func = c->func;
 		micro_hash_foreach_dep (deprange->deps, dep,
 			(func) (dep, c->user););
 	}
 }
 
 static void
-cell_foreach_range_dep (GnmCell const *cell, DepFunc func, gpointer user)
+cell_foreach_range_dep (GnmCell const *cell, GnmDepFunc func, gpointer user)
 {
 	search_rangedeps_closure_t closure;
 	GHashTable *bucket =
@@ -1703,7 +1679,7 @@ cell_foreach_range_dep (GnmCell const *cell, DepFunc func, gpointer user)
 
 static void
 cell_foreach_single_dep (Sheet const *sheet, int col, int row,
-			 DepFunc func, gpointer user)
+			 GnmDepFunc func, gpointer user)
 {
 	DependencySingle lookup, *single;
 	GnmDepContainer *deps = sheet->deps;
@@ -1717,8 +1693,15 @@ cell_foreach_single_dep (Sheet const *sheet, int col, int row,
 			(*func) (dep, user););
 }
 
+/**
+ * cell_foreach_dep:
+ * @cell: #GnmCell
+ * @func: (scope call):
+ * @user: user data.
+ *
+ **/
 void
-cell_foreach_dep (GnmCell const *cell, DepFunc func, gpointer user)
+cell_foreach_dep (GnmCell const *cell, GnmDepFunc func, gpointer user)
 {
 	g_return_if_fail (cell != NULL);
 
@@ -1922,8 +1905,8 @@ dependents_unrelocate (GSList *info)
 }
 
 /**
- * dependents_link :
- * @deps : An slist of dependents.
+ * dependents_link:
+ * @deps: (element-type GnmDependent): An slist of dependents.
  *
  * link a list of dependents.  (The list used to get freed, but is not
  * freed anymore.)
@@ -2042,10 +2025,10 @@ names_referencing_sheet (Sheet *sheet)
 
 /**
  * dependents_relocate:
- * @rinfo : the descriptor record for what is being moved where.
+ * @info: the descriptor record for what is being moved where.
  *
  * Fixes references to or from a region that is going to be moved.
- * Returns a list of the locations and expressions that were changed outside of
+ * Returns: (transfer full): a list of the locations and expressions that were changed outside of
  * the region.
  **/
 GOUndo *
@@ -2404,7 +2387,7 @@ handle_referencing_names (GnmDepContainer *deps, Sheet *sheet)
 static void
 handle_outgoing_references (GnmDepContainer *deps, Sheet *sheet)
 {
-	DependentFlags what = DEPENDENT_USES_NAME;
+	GnmDependentFlags what = DEPENDENT_USES_NAME;
 	GSList *accum = NULL;
 
 	what |= (sheet->workbook && sheet->workbook->during_destruction)
@@ -2737,13 +2720,19 @@ workbook_recalc (Workbook *wb)
  * workbook_recalc_all :
  * @wb :
  *
- * Queues all dependents for recalc and marks them all as dirty.
+ * Queues all dependents for recalc and recalculates.
  */
 void
 workbook_recalc_all (Workbook *wb)
 {
 	workbook_queue_all_recalc (wb);
+
+	/* Recalculate this workbook unconditionally.  */
 	workbook_recalc (wb);
+
+	/* Recalculate other workbooks when needed.  */
+	gnm_app_recalc ();
+
 	WORKBOOK_FOREACH_VIEW (wb, view,
 		sheet_update (wb_view_cur_sheet (view)););
 }
@@ -2766,7 +2755,8 @@ dynamic_dep_free (DynamicDep *dyn)
 
 	for (ptr = dyn->ranges ; ptr != NULL ; ptr = ptr->next) {
 		rr = ptr->data;
-		unlink_cellrange_dep (&dyn->base, pos, &rr->a, &rr->b);
+		link_unlink_cellrange_dep (&dyn->base, pos,
+					   &rr->a, &rr->b, FALSE);
 		g_free (rr);
 	}
 	g_slist_free (dyn->ranges);
@@ -2777,6 +2767,12 @@ dynamic_dep_free (DynamicDep *dyn)
 	g_free (dyn);
 }
 
+/**
+ * gnm_dep_container_new: (skip)
+ * @sheet: #Sheet
+ *
+ * Returns: (transfer full):
+ **/
 GnmDepContainer *
 gnm_dep_container_new (Sheet *sheet)
 {
@@ -2835,10 +2831,39 @@ static void
 dependent_debug_name_for_sheet (GnmDependent const *dep, Sheet *sheet,
 				GString *target)
 {
-	if (sheet && sheet == dep->sheet && dependent_is_cell (dep))
-		g_string_append (target, cell_name (GNM_DEP_TO_CELL (dep)));
-	else
-		dependent_debug_name (dep, target);
+	int t;
+	GnmDependentClass *klass;
+
+	g_return_if_fail (dep != NULL);
+	g_return_if_fail (dep_classes);
+
+	if (!dep->sheet)
+		g_warning ("Invalid dep, missing sheet");
+
+	if (sheet == dep->sheet) {
+		/* Nothing */
+	} else {
+		g_string_append (target,
+				 dep->sheet ? dep->sheet->name_quoted : "?");
+		g_string_append_c (target, '!');
+	}
+
+	t = dependent_type (dep);
+	klass = g_ptr_array_index (dep_classes, t);
+	klass->debug_name (dep, target);
+}
+
+
+/**
+ * dependent_debug_name :
+ * @dep : The dependent we are interested in.
+ *
+ * A useful little debugging utility.
+ */
+void
+dependent_debug_name (GnmDependent const *dep, GString *target)
+{
+	dependent_debug_name_for_sheet (dep, NULL, target);
 }
 
 
@@ -3044,34 +3069,4 @@ gnm_dep_container_sanity_check (GnmDepContainer const *deps)
 	g_hash_table_destroy (seenb4);
 }
 
-/**
- * dependent_debug_name :
- * @dep : The dependent we are interested in.
- *
- * A useful little debugging utility.
- */
-void
-dependent_debug_name (GnmDependent const *dep, GString *target)
-{
-	int t;
-
-	g_return_if_fail (dep != NULL);
-	g_return_if_fail (dep_classes);
-
-	if (dep->sheet != NULL) {
-		g_string_append (target, dep->sheet->name_quoted);
-		g_string_append_c (target, '!');
-	} else
-		g_warning ("Invalid dep, missing sheet");
-
-	t = dependent_type (dep);
-	if (t != DEPENDENT_CELL) {
-		GnmDependentClass *klass = g_ptr_array_index (dep_classes, t);
-
-		g_return_if_fail (klass);
-		klass->debug_name (dep, target);
-	} else
-		g_string_append (target, cell_name (GNM_DEP_TO_CELL (dep)));
-}
-
 /* ------------------------------------------------------------------------- */
