@@ -532,6 +532,32 @@ gnm_expr_free (GnmExpr const *expr)
 	}
 }
 
+GType
+gnm_expr_get_type (void)
+{
+	static GType t = 0;
+
+	if (t == 0) {
+		t = g_boxed_type_register_static ("GnmExpr",
+			 (GBoxedCopyFunc)gnm_expr_copy,
+			 (GBoxedFreeFunc)gnm_expr_free);
+	}
+	return t;
+}
+
+GType
+gnm_expr_array_corner_get_type (void)
+{
+	static GType t = 0;
+
+	if (t == 0) {
+		t = g_boxed_type_register_static ("GnmExprArrayCorner",
+			 (GBoxedCopyFunc)gnm_expr_copy,
+			 (GBoxedFreeFunc)gnm_expr_free);
+	}
+	return t;
+}
+
 /**
  * gnm_expr_equal : Returns TRUE if the supplied expressions are exactly the
  *   same.  No eval position is used to see if they are effectively the same.
@@ -1580,7 +1606,7 @@ gnm_expr_simplify_if (GnmExpr const *expr)
 	if (expr->func.func != f_if || expr->func.argc != 3)
 		return NULL;
 
-	cond = expr->func.argv[0];	
+	cond = expr->func.argv[0];
 	if (GNM_EXPR_GET_OPER (cond) == GNM_EXPR_OP_CONSTANT) {
 		GnmValue const *condval = cond->constant.value;
 		gboolean err;
@@ -1730,12 +1756,12 @@ do_expr_as_string (GnmExpr const *expr, int paren_level,
 			return;
 		}
 
-		if (v->type == VALUE_BOOLEAN && 
+		if (v->type == VALUE_BOOLEAN &&
 		    out->convs->output.boolean != NULL) {
 			out->convs->output.boolean (out, v->v_bool.val);
 			return;
 		}
-		
+
 
 		value_get_as_gstring (v, target, out->convs);
 
@@ -2814,16 +2840,39 @@ gnm_expr_sharer_new (void)
 		 (GEqualFunc)gnm_expr_top_equal,
 		 (GDestroyNotify)gnm_expr_top_unref,
 		 NULL);
+	es->ref_count = 1;
 	return es;
 }
 
 void
 gnm_expr_sharer_destroy (GnmExprSharer *es)
 {
+	if (!es || es->ref_count-- > 1)
+		return;
 	g_hash_table_destroy (es->exprs);
 	g_free (es);
 }
 
+static GnmExprSharer *
+gnm_expr_sharer_ref (GnmExprSharer *es)
+{
+	es->ref_count++;
+	return es;
+}
+
+GType
+gnm_expr_sharer_get_type (void)
+{
+	static GType t = 0;
+
+	if (t == 0) {
+		t = g_boxed_type_register_static ("GnmExprSharer",
+			 (GBoxedCopyFunc)gnm_expr_sharer_ref,
+			 (GBoxedFreeFunc)gnm_expr_sharer_destroy);
+	}
+	return t;
+}
+
 GnmExprTop const *
 gnm_expr_sharer_share (GnmExprSharer *es, GnmExprTop const *texpr)
 {
@@ -2907,6 +2956,19 @@ gnm_expr_top_unref (GnmExprTop const *texpr)
 	}
 }
 
+GType
+gnm_expr_top_get_type (void)
+{
+	static GType t = 0;
+
+	if (t == 0) {
+		t = g_boxed_type_register_static ("GnmExprTop",
+			 (GBoxedCopyFunc)gnm_expr_top_ref,
+			 (GBoxedFreeFunc)gnm_expr_top_unref);
+	}
+	return t;
+}
+
 gboolean
 gnm_expr_top_is_shared (GnmExprTop const *texpr)
 {
@@ -2932,8 +2994,8 @@ gnm_expr_top_new_array_elem  (int x, int y)
  * @texpr :
  *
  * A collect the set of GnmRanges in @expr.
- * Return a list of the unique references Caller is responsible for releasing
- * the list and the content.
+ * Return: (element-type GnmRange) (transfer full): a list of the unique
+ * references Caller is responsible for releasing the list and the content.
  **/
 GSList *
 gnm_expr_top_get_ranges (GnmExprTop const *texpr)
@@ -3111,12 +3173,13 @@ gnm_expr_top_eval (GnmExprTop const *texpr,
 }
 
 /**
- * gnm_expr_top_referenced_sheets :
- * @texpr :
+ * gnm_expr_top_referenced_sheets:
+ * @texpr:
  *
  * Generates a list of the sheets referenced by the supplied expression.
  * Caller must free the list.  Note, that NULL may occur in the result
  * if the expression has a range or cellref without a sheet.
+ * Returns: (element-type Sheet) (transfer container): the created list.
  */
 GSList *
 gnm_expr_top_referenced_sheets (GnmExprTop const *texpr)
@@ -3197,6 +3260,12 @@ gnm_expr_top_is_rangeref (GnmExprTop const *texpr)
 	return gnm_expr_is_rangeref (texpr->expr);
 }
 
+/**
+ * gnm_expr_top_new_array_corner:
+ * @expr: #GnmExprTop
+ *
+ * Returns: (transfer none): the #GnmExprArrayCorner or %NULL.
+ **/
 GnmExprArrayCorner const *
 gnm_expr_top_get_array_corner (GnmExprTop const *texpr)
 {
