@@ -281,6 +281,8 @@ main (int argc, char const **argv)
 		return gnm_dump_func_defs (ext_refs_file, 4);
 
 	if (with_gui) {
+		go_component_set_default_command_context (cc = cmd_context_stderr_new ());
+		g_object_unref (cc);
 		cc = g_object_new (GNM_TYPE_IO_CONTEXT_GTK,
 				   "show-splash", !gnumeric_no_splash,
 				   "show-warnings", !gnumeric_no_warnings,
@@ -292,8 +294,8 @@ main (int argc, char const **argv)
 		/* TODO: Make this inconsistency go away */
 		cc = cmd_context_stderr_new ();
 		ioc = go_io_context_new (cc);
+		go_component_set_default_command_context (cc);
 	}
-	go_component_set_command_context (cc);
 
 	/* Keep in sync with .desktop file */
 	g_set_application_name (_("Gnumeric Spreadsheet"));
@@ -317,7 +319,7 @@ main (int argc, char const **argv)
 			}
 
 			go_io_context_processing_file (ioc, uri);
-			wbv = wb_view_new_from_uri (uri, NULL, ioc, NULL);
+			wbv = workbook_view_new_from_uri (uri, NULL, ioc, NULL);
 			g_free (uri);
 
 			if (go_io_error_occurred (ioc) ||
@@ -328,7 +330,7 @@ main (int argc, char const **argv)
 			if (wbv != NULL) {
 				WBCGtk *wbcg;
 
-				workbook_update_history (wb_view_get_workbook (wbv));
+				workbook_update_history (wb_view_get_workbook (wbv), FILE_SAVE_AS_SAVE);
 
 				wbcg = wbc_gtk_new (wbv, NULL, NULL, geometry);
 				geometry = NULL;
@@ -369,7 +371,6 @@ main (int argc, char const **argv)
 			for (l = wbcgs_to_kill; l; l = l->next)
 				g_idle_add ((GSourceFunc)cb_kill_wbcg, l->data);
 		}
-		g_object_unref (ioc);
 
 		g_signal_connect (gnm_app_get_app (),
 				  "workbook_removed",
@@ -377,6 +378,7 @@ main (int argc, char const **argv)
 				  NULL);
 
 		gnm_io_context_gtk_discharge_splash (GNM_IO_CONTEXT_GTK (ioc));
+		g_object_unref (ioc);
 
 		g_idle_add ((GSourceFunc)pathetic_qt_workaround, NULL);
 		gtk_main ();
@@ -390,8 +392,6 @@ main (int argc, char const **argv)
 	store_plugin_state ();
 	gnm_shutdown ();
 
-	go_component_set_command_context (NULL);
-
 #if defined(G_OS_WIN32)
 	if (has_console) {
 		close(1);
@@ -401,6 +401,7 @@ main (int argc, char const **argv)
 #endif
 
 	gnm_pre_parse_shutdown ();
+	go_component_set_default_command_context (NULL);
 
 	/*
 	 * This helps finding leaks.  We might want it in developent
