@@ -183,11 +183,13 @@ sheet_style_unlink (Sheet *sheet, GnmStyle *st)
 /**
  * sheet_style_find :
  *
- * @sheet : the sheet
- * @s     : a style
+ * @sheet: (transfer full): the sheet
+ * @s: a style
  *
  * Looks up a style from the sheets collection.  Linking if necessary.
  * ABSORBS the reference and adds a link.
+ *
+ * Returns: (transfer full): the new style.
  */
 GnmStyle *
 sheet_style_find (Sheet const *sheet, GnmStyle *s)
@@ -656,7 +658,7 @@ sheet_style_init_size (Sheet *sheet, int cols, int rows)
 	 * 'none' by default.  Then displays that as white. */
 	if (sheet->sheet_type == GNM_SHEET_OBJECT) {
 		gnm_style_set_back_color (default_style,
-			style_color_new_i8 (0x50, 0x50, 0x50));
+			gnm_color_new_rgb8 (0x50, 0x50, 0x50));
 		gnm_style_set_pattern (default_style, 1);
 	}
 #endif
@@ -1240,11 +1242,10 @@ tail_recursion :
 }
 
 /**
- * sheet_style_set_range
- *
- * @sheet :
- * @range :
- * @style : #GnmStyle
+ * sheet_style_set_range:
+ * @sheet:
+ * @range:
+ * @style: #GnmStyle
  *
  * Change the complete style for a region.
  * This function absorbs a reference to the new @style.
@@ -1265,10 +1266,10 @@ sheet_style_set_range (Sheet *sheet, GnmRange const *range,
 }
 
 /**
- * sheet_style_apply_col
- * @sheet :
- * @col :
- * @pstyle : #GnmStyle
+ * sheet_style_apply_col:
+ * @sheet:
+ * @col:
+ * @style: #GnmStyle
  *
  * NOTE : This is a simple wrapper for now.  When we support col/row styles it
  *	will make life easier.
@@ -1285,10 +1286,10 @@ sheet_style_apply_col (Sheet *sheet, int col, GnmStyle *pstyle)
 }
 
 /**
- * sheet_style_apply_row
- * @sheet :
- * @row :
- * @pstyle : #GnmStyle
+ * sheet_style_apply_row:
+ * @sheet:
+ * @row:
+ * @style: #GnmStyle
  *
  * NOTE : This is a simple wrapper for now.  When we support col/row styles it
  *	will make life easier.
@@ -1775,7 +1776,7 @@ typedef struct {
 
 static void
 cb_find_conflicts (GnmStyle *style,
-		   G_GNUC_UNUSED int corner_col, G_GNUC_UNUSED int corner_row, 
+		   G_GNUC_UNUSED int corner_col, G_GNUC_UNUSED int corner_row,
 		   G_GNUC_UNUSED int width, G_GNUC_UNUSED int height,
 		   G_GNUC_UNUSED GnmRange const *apply_to, FindConflicts *ptr)
 {
@@ -2065,7 +2066,7 @@ cb_style_extent (GnmStyle *style,
  * sheet_style_get_extent:
  *
  * @sheet: sheet to measure
- * @res: starting range and resulting range
+ * @r: starting range and resulting range
  *
  * A simple implementation that finds the smallest range containing all visible styles
  * and containing @res.
@@ -2132,7 +2133,7 @@ struct cb_is_default {
 
 static void
 cb_is_default (GnmStyle *style,
-	       int corner_col, G_GNUC_UNUSED int corner_row, 
+	       int corner_col, G_GNUC_UNUSED int corner_row,
 	       int width, G_GNUC_UNUSED int height,
 	       GnmRange const *apply_to, gpointer user_)
 {
@@ -2240,6 +2241,13 @@ sheet_style_most_common (Sheet const *sheet, gboolean is_col)
 
 /****************************************************************************/
 
+/**
+ * gnm_style_region_new:
+ * @range: #GnmRange
+ * @style: #GnmStyle
+ *
+ * Returns: (transfer full): the newly allocated #GnmStyleRegion.
+ **/
 GnmStyleRegion *
 gnm_style_region_new (GnmRange const *range, GnmStyle *style)
 {
@@ -2263,6 +2271,29 @@ gnm_style_region_free (GnmStyleRegion *sr)
 	g_free (sr);
 }
 
+static GnmStyleRegion *
+gnm_style_region_copy (GnmStyleRegion *sr)
+{
+	GnmStyleRegion *res = g_new (GnmStyleRegion, 1);
+	*res = *sr;
+	gnm_style_ref (sr->style);
+	return res;
+}
+
+GType
+gnm_style_region_get_type (void)
+{
+	static GType t = 0;
+
+	if (t == 0) {
+		t = g_boxed_type_register_static ("GnmStyleRegion",
+			 (GBoxedCopyFunc)gnm_style_region_copy,
+			 (GBoxedFreeFunc)gnm_style_region_free);
+	}
+	return t;
+}
+
+
 typedef struct {
 	GHashTable *cache;
 	gboolean (*style_equal) (GnmStyle const *a, GnmStyle const *b);
@@ -2385,13 +2416,14 @@ cb_hash_to_list (G_GNUC_UNUSED gpointer key, gpointer	value, gpointer	user_data)
 }
 
 /**
- * sheet_style_get_range :
- *
- * @sheet :
- * @range :
+ * sheet_style_get_range:
+ * @sheet:
+ * @range:
  *
  * Get a list of rectangles and their associated styles
  * Caller is responsible for freeing.
+ *
+ * Returns: (transfer full):
  */
 GnmStyleList *
 sheet_style_get_range (Sheet const *sheet, GnmRange const *r)
@@ -2444,11 +2476,11 @@ style_conditions_equal (GnmStyle const *a, GnmStyle const *b)
 
 /**
  * sheet_style_collect_conditions:
- * @sheet :
- * @range :
+ * @sheet:
+ * @range:
  *
- * Returns a list of areas with conditionals, Caller is responsible for
- * freeing.
+ * Returns: (transfer full): a list of areas with conditionals, Caller is
+ * responsible for freeing.
  **/
 GnmStyleList *
 sheet_style_collect_conditions (Sheet const *sheet, GnmRange const *r)
@@ -2494,11 +2526,12 @@ style_hlink_equal (GnmStyle const *a, GnmStyle const *b)
 }
 
 /**
- * sheet_style_collect_hlinks :
- * @sheet :
- * @range :
+ * sheet_style_collect_hlinks:
+ * @sheet:
+ * @range:
  *
- * Returns a list of areas with hyperlinks, Caller is responsible for freeing.
+ * Returns: (transfer full): a list of areas with hyperlinks, Caller is
+ * responsible for freeing.
  **/
 GnmStyleList *
 sheet_style_collect_hlinks (Sheet const *sheet, GnmRange const *r)
@@ -2548,11 +2581,12 @@ style_validation_equal (GnmStyle const *a, GnmStyle const *b)
 }
 
 /**
- * sheet_style_collect_validations :
- * @sheet :
- * @range :
+ * sheet_style_collect_validations:
+ * @sheet:
+ * @range:
  *
- * Returns a list of areas with validation, Caller is responsible for freeing.
+ * Returns: (transfer full): a list of areas with validation, Caller is
+ * responsible for freeing.
  **/
 GnmStyleList *
 sheet_style_collect_validations (Sheet const *sheet, GnmRange const *r)
@@ -2582,11 +2616,12 @@ sheet_style_collect_validations (Sheet const *sheet, GnmRange const *r)
 }
 
 /**
- * sheet_style_set_list
- * @sheet     : #Sheet
- * @corner    : The top-left corner (in LTR mode)
- * @transpose : should the styles be transposed
- * @list      : #GnmStyleList
+ * sheet_style_set_list:
+ * @sheet: #Sheet
+ * @corner: The top-left corner (in LTR mode)
+ * @l: #GnmStyleList
+ * @range_modify: (scope call):
+ * @data: user data
  *
  * Overwrites the styles of the ranges given by @corner with the content of
  * @list. Optionally transposing the ranges
@@ -2663,7 +2698,7 @@ style_list_get_style (GnmStyleList const *list, int col, int row)
 
 static void
 cb_find_link (GnmStyle *style,
-	      G_GNUC_UNUSED int corner_col, G_GNUC_UNUSED int corner_row, 
+	      G_GNUC_UNUSED int corner_col, G_GNUC_UNUSED int corner_row,
 	      G_GNUC_UNUSED int width, G_GNUC_UNUSED int height,
 	      G_GNUC_UNUSED GnmRange const *apply_to, gpointer user)
 {
@@ -2673,12 +2708,14 @@ cb_find_link (GnmStyle *style,
 }
 
 /**
- * sheet_style_region_contains_link :
- * @sheet :
- * @r :
+ * sheet_style_region_contains_link:
+ * @sheet:
+ * @r:
  *
  * Utility routine that checks to see if a region contains at least 1 hyper link
  * and returns the 1st one it finds.
+ *
+ * Returns: (transfer none): the found #GmHLink if any.
  **/
 GnmHLink *
 sheet_style_region_contains_link (Sheet const *sheet, GnmRange const *r)
@@ -2694,6 +2731,14 @@ sheet_style_region_contains_link (Sheet const *sheet, GnmRange const *r)
 	return res;
 }
 
+/**
+ * sheet_style_foreach:
+ * @sheet: #Sheet
+ * @func: (scope call): callback
+ * @user_data: user data.
+ *
+ * Executes @func for each style in the sheet.
+ **/
 void
 sheet_style_foreach (Sheet const *sheet, GHFunc func, gpointer user_data)
 {
@@ -2730,6 +2775,14 @@ cb_hash_to_cb (G_GNUC_UNUSED gpointer key, gpointer value, gpointer user_data)
 
 }
 
+/**
+ * sheet_style_range_foreach:
+ * @sheet: #Sheet
+ * @func: (scope call): callback.
+ * @user_data: user data
+ * @optimize:
+ *
+ **/
 void
 sheet_style_range_foreach (Sheet const *sheet, GHFunc func, gpointer user_data, gboolean optimize)
 {
