@@ -215,12 +215,13 @@ resize_columns (Sheet *sheet)
  **/
 static void
 stf_read_workbook (G_GNUC_UNUSED GOFileOpener const *fo,  gchar const *enc,
-		   GOIOContext *context, gpointer wbv, GsfInput *input)
+		   GOIOContext *context, GoView *view, GsfInput *input)
 {
 	DialogStfResult_t *dialogresult = NULL;
 	char *name, *nameutf8 = NULL;
 	char *data = NULL;
 	size_t data_len;
+	WorkbookView *wbv = WORKBOOK_VIEW (view);
 
 	/* FIXME : how to do this cleanly ? */
 	if (!IS_WBC_GTK (context->impl))
@@ -296,8 +297,9 @@ cb_get_content (GnmCellIter const *iter, GsfOutput *buf)
 }
 
 /**
- * stf_text_to_columns
- * @wbc  : The control making the request
+ * stf_text_to_columns:
+ * @wbc: The control making the request
+ * @cc:
  *
  * Main routine, handles importing a file including all dialog mumbo-jumbo
  **/
@@ -416,9 +418,9 @@ clear_stray_NULs (GOIOContext *context, GString *utf8data)
 static void
 stf_read_workbook_auto_csvtab (G_GNUC_UNUSED GOFileOpener const *fo, gchar const *enc,
 			       GOIOContext *context,
-			       gpointer wbv, GsfInput *input)
+			       GoView *view, GsfInput *input)
 {
-	Sheet *sheet, *old_sheet;
+	Sheet *sheet;
 	Workbook *book;
 	char *name;
 	char *data;
@@ -429,12 +431,12 @@ stf_read_workbook_auto_csvtab (G_GNUC_UNUSED GOFileOpener const *fo, gchar const
 	int cols, rows, i;
 	GStringChunk *lines_chunk;
 	GPtrArray *lines;
+	WorkbookView *wbv = WORKBOOK_VIEW (view);
 
 	g_return_if_fail (context != NULL);
 	g_return_if_fail (wbv != NULL);
 
 	book = wb_view_get_workbook (wbv);
-	old_sheet = wb_view_cur_sheet (wbv);
 
 	data = stf_preparse (context, input, &data_len);
 	if (!data)
@@ -485,6 +487,7 @@ stf_read_workbook_auto_csvtab (G_GNUC_UNUSED GOFileOpener const *fo, gchar const
 	workbook_sheet_attach (book, sheet);
 
 	if (stf_parse_sheet (po, utf8data->str, NULL, sheet, 0, 0)) {
+		gboolean is_csv;
 		workbook_recalc_all (book);
 		resize_columns (sheet);
 		if (po->cols_exceeded || po->rows_exceeded) {
@@ -492,11 +495,12 @@ stf_read_workbook_auto_csvtab (G_GNUC_UNUSED GOFileOpener const *fo, gchar const
 				     _("Some data did not fit on the "
 				       "sheet and was dropped."));
 		}
+		is_csv = po->sep.chr[0] == ',';
 		workbook_set_saveinfo
 			(book,
 			 GO_FILE_FL_WRITE_ONLY,
 			 go_file_saver_for_id
-			 ("Gnumeric_stf:stf_assistant"));
+			 (is_csv ? "Gnumeric_stf:stf_csv" : "Gnumeric_stf:stf_assistant"));
 	} else {
 		workbook_sheet_delete (sheet);
 		go_cmd_context_error_import (GO_CMD_CONTEXT (context),
@@ -512,10 +516,11 @@ stf_read_workbook_auto_csvtab (G_GNUC_UNUSED GOFileOpener const *fo, gchar const
 
 static void
 stf_write_csv (G_GNUC_UNUSED GOFileSaver const *fs, GOIOContext *context,
-	       gconstpointer wbv, GsfOutput *output)
+	       GoView const *view, GsfOutput *output)
 {
 	Sheet *sheet;
 	GnmRangeRef const *range;
+	WorkbookView *wbv = WORKBOOK_VIEW (view);
 
 	GnmStfExport *config = g_object_new
 		(GNM_STF_EXPORT_TYPE,
@@ -646,7 +651,7 @@ stf_init (void)
 	go_file_opener_register (opener, 0);
 	g_object_unref (opener);
 
-	saver = gnm_stf_file_saver_new ("Gnumeric_stf:stf_assistant");
+	saver = gnm_stf_file_saver_create ("Gnumeric_stf:stf_assistant");
 	go_file_saver_register (saver);
 	g_object_unref (saver);
 
