@@ -113,7 +113,7 @@ style_border_equal (gconstpointer v1, gconstpointer v2)
 
 	/*
 	 * ->color is a pointer, but the comparison is safe because
-	 * all colours are cached, see style_color_new_i16.
+	 * all colours are cached, see gnm_color_new_go.
 	 */
 	return	(k1->color == k2->color) &&
 		(k1->line_type == k2->line_type);
@@ -128,7 +128,7 @@ style_border_hash (gconstpointer v)
 	 * HACK ALERT!
 	 *
 	 * ->color is a pointer, but the comparison is safe because
-	 * all colours are cached, see style_color_new_i16.
+	 * all colours are cached, see gnm_color_new_go.
 	 *
 	 */
 	return (GPOINTER_TO_UINT(b->color) ^ b->line_type);
@@ -317,6 +317,19 @@ gnm_style_border_unref (GnmBorder *border)
 	g_free (border);
 }
 
+GType
+gnm_border_get_type (void)
+{
+	static GType t = 0;
+
+	if (t == 0) {
+		t = g_boxed_type_register_static ("GnmBorder",
+			 (GBoxedCopyFunc)gnm_style_border_ref,
+			 (GBoxedFreeFunc)gnm_style_border_unref);
+	}
+	return t;
+}
+
 static gboolean
 style_border_hmargins (GnmBorder const * const * prev_vert,
 		       GnmStyleRow const *sr, int col,
