@@ -93,10 +93,10 @@ enum {
 };
 
 /**
- * wb_view_get_workbook :
- * @wbv : #WorkbookView
+ * wb_view_get_workbook:
+ * @wbv: #WorkbookView
  *
- * Return the #Workbook associated with @wbv
+ * Returns: (transfer none): the #Workbook associated with @wbv
  **/
 Workbook *
 wb_view_get_workbook (WorkbookView const *wbv)
@@ -106,10 +106,10 @@ wb_view_get_workbook (WorkbookView const *wbv)
 }
 
 /**
- * wb_view_get_doc :
- * @wbv : #WorkbookView
+ * wb_view_get_doc:
+ * @wbv: #WorkbookView
  *
- * Return the #Workbook associated with @wbv cast to a #GODoc
+ * Returns: (transfer none): the #Workbook associated with @wbv cast to a #GODoc
  **/
 GODoc *
 wb_view_get_doc (WorkbookView const *wbv)
@@ -137,6 +137,12 @@ wb_view_get_index_in_wb (WorkbookView const *wbv)
 	return -1;
 }
 
+/**
+ * wb_view_cur_sheet:
+ * @wbv: #WorkbookView
+ *
+ * Returns: (transfer none): the current sheet.
+ **/
 Sheet *
 wb_view_cur_sheet (WorkbookView const *wbv)
 {
@@ -144,6 +150,12 @@ wb_view_cur_sheet (WorkbookView const *wbv)
 	return wbv->current_sheet;
 }
 
+/**
+ * wb_view_cur_sheet_view:
+ * @wbv: #WorkbookView
+ *
+ * Returns: (transfer none): the current sheet view.
+ **/
 SheetView *
 wb_view_cur_sheet_view (WorkbookView const *wbv)
 {
@@ -969,7 +981,7 @@ workbook_view_class_init (GObjectClass *gobject_class)
 }
 
 GSF_CLASS (WorkbookView, workbook_view,
-	   workbook_view_class_init, NULL, G_TYPE_OBJECT)
+	   workbook_view_class_init, NULL, GO_TYPE_VIEW)
 
 WorkbookView *
 workbook_view_new (Workbook *wb)
@@ -1015,11 +1027,11 @@ workbook_view_new (Workbook *wb)
 }
 
 /**
- * wbv_save_to_output :
- * @wbv : #WorkbookView
- * @fs  : #GOFileSaver
- * @output : #GsfOutput
- * @io_context : #GOIOContext
+ * wbv_save_to_output:
+ * @wbv: #WorkbookView
+ * @fs: #GOFileSaver
+ * @output: #GsfOutput
+ * @io_context: #GOIOContext
  *
  * NOTE : Temporary api until we get the new output framework.
  **/
@@ -1034,7 +1046,7 @@ wbv_save_to_output (WorkbookView *wbv, GOFileSaver const *fs,
 	if (go_doc_is_dirty (godoc))
 	  /* FIXME: we should be using the true modification time */
 	  gnm_insert_meta_date (godoc, GSF_META_NAME_DATE_MODIFIED);
-	go_file_saver_save (fs, io_context, wbv, output);
+	go_file_saver_save (fs, io_context, GO_VIEW (wbv), output);
 
 	/* The plugin convention is unclear */
 	if (!gsf_output_is_closed (output))
@@ -1078,10 +1090,10 @@ wb_view_save_to_uri (WorkbookView *wbv, GOFileSaver const *fs,
 
 /**
  * wb_view_save_as:
- * @wbv         : Workbook View
- * @fs          : GOFileSaver object
- * @uri         : URI to save as.
- * @context     :
+ * @wbv: Workbook View
+ * @fs: GOFileSaver object
+ * @file_name: URI to save as.
+ * @context:
  *
  * Saves @wbv and workbook it's attached to into @uri file using
  * @fs file saver.  If the format sufficiently advanced make it the saver
@@ -1113,13 +1125,15 @@ wb_view_save_as (WorkbookView *wbv, GOFileSaver *fs, char const *uri,
 	has_error   = go_io_error_occurred (io_context);
 	has_warning = go_io_warning_occurred (io_context);
 	if (!has_error) {
-		if (workbook_set_saveinfo (wb,
-			go_file_saver_get_format_level (fs), fs) &&
-		    go_doc_set_uri (GO_DOC (wb), uri)) {
-			go_doc_set_dirty (GO_DOC (wb), FALSE);
-			/* See 634792.  */
-			go_doc_set_pristine (GO_DOC (wb), FALSE);
-		}
+		if (workbook_set_saveinfo
+		    (wb, go_file_saver_get_format_level (fs), fs)) {
+			if (go_doc_set_uri (GO_DOC (wb), uri)) {
+				go_doc_set_dirty (GO_DOC (wb), FALSE);
+				/* See 634792.  */
+				go_doc_set_pristine (GO_DOC (wb), FALSE);
+			}
+		} else
+			workbook_set_last_export_uri (wb, g_strdup (uri));
 	}
 	if (has_error || has_warning)
 		go_io_error_display (io_context);
@@ -1131,8 +1145,8 @@ wb_view_save_as (WorkbookView *wbv, GOFileSaver *fs, char const *uri,
 
 /**
  * wb_view_save:
- * @wbv         : The view to save.
- * @context     : The context that invoked the operation
+ * @wbv: The view to save.
+ * @cc: The context that invoked the operation
  *
  * Saves @wbv and workbook it's attached to into file assigned to the
  * workbook using workbook's file saver. If the workbook has no file
@@ -1180,12 +1194,25 @@ wb_view_save (WorkbookView *wbv, GOCmdContext *context)
 	return !has_error;
 }
 
+/**
+ * workbook_view_new_from_uri:
+ * @input: #GsfInput to read data from.
+ * @optional_format: Optional GOFileOpener
+ * @io_context: Optional context to display errors.
+ * @optional_encoding: Optional encoding for GOFileOpener that understand it
+ *
+ * Reads @uri file using given file opener @optional_fmt, or probes for a valid
+ * possibility if @optional_fmt is NULL.  Reports problems to @io_context.
+ *
+ * Return value: (transfer full): the newly allocated WorkbookView or %NULL
+ * on error.
+ **/
 WorkbookView *
-wb_view_new_from_input (GsfInput *input,
-			const char *optional_uri,
-			GOFileOpener const *optional_fmt,
-			GOIOContext *io_context,
-			char const *optional_enc)
+workbook_view_new_from_input (GsfInput *input,
+                              const char *optional_uri,
+                              GOFileOpener const *optional_fmt,
+                              GOIOContext *io_context,
+                              char const *optional_enc)
 {
 	WorkbookView *new_wbv = NULL;
 
@@ -1241,7 +1268,8 @@ wb_view_new_from_input (GsfInput *input,
 
 		/* disable recursive dirtying while loading */
 		old = workbook_enable_recursive_dirty (new_wb, FALSE);
-		go_file_opener_open (optional_fmt, optional_enc, io_context, new_wbv, input);
+		go_file_opener_open (optional_fmt, optional_enc, io_context,
+		                     GO_VIEW (new_wbv), input);
 		workbook_enable_recursive_dirty (new_wb, old);
 
 		if (go_io_error_occurred (io_context)) {
@@ -1257,6 +1285,9 @@ wb_view_new_from_input (GsfInput *input,
 			workbook_optimize_style (new_wb);
 			workbook_recalc (new_wb);
 			go_doc_set_dirty (GO_DOC (new_wb), FALSE);
+			if (optional_uri && workbook_get_file_exporter (new_wb))
+				workbook_set_last_export_uri
+					(new_wb, g_strdup (optional_uri));
 		}
 	} else
 		go_cmd_context_error_import (GO_CMD_CONTEXT (io_context),
@@ -1266,19 +1297,20 @@ wb_view_new_from_input (GsfInput *input,
 }
 
 /**
- * wb_view_new_from_uri :
- * @uri          : URI for file
- * @optional_fmt : Optional GOFileOpener
- * @io_context   : Optional context to display errors.
- * @optional_enc : Optional encoding for GOFileOpener that understand it
+ * workbook_view_new_from_uri:
+ * @uri: URI for file
+ * @optional_format: Optional GOFileOpener
+ * @io_context: Optional context to display errors.
+ * @optional_encoding: Optional encoding for GOFileOpener that understand it
  *
  * Reads @uri file using given file opener @optional_fmt, or probes for a valid
  * possibility if @optional_fmt is NULL.  Reports problems to @io_context.
  *
- * Return value: TRUE if file was successfully read and FALSE otherwise.
- */
+ * Return value: (transfer full): the newly allocated WorkbookView or %NULL
+ * on error.
+ **/
 WorkbookView *
-wb_view_new_from_uri (char const *uri,
+workbook_view_new_from_uri (char const *uri,
 		      GOFileOpener const *optional_fmt,
 		      GOIOContext *io_context,
 		      char const *optional_enc)
@@ -1296,7 +1328,7 @@ wb_view_new_from_uri (char const *uri,
 #if 0
 		g_printerr ("Reading %s\n", uri);
 #endif
-		res = wb_view_new_from_input (input, uri,
+		res = workbook_view_new_from_input (input, uri,
 					      optional_fmt, io_context,
 					      optional_enc);
 		g_object_unref (G_OBJECT (input));
