@@ -75,19 +75,33 @@ cb_exporter_finalize (Workbook *wb, GOFileSaver *saver)
 	g_return_if_fail (GO_IS_FILE_SAVER (saver));
 	g_return_if_fail (IS_WORKBOOK (wb));
 	g_return_if_fail (wb->file_exporter == saver);
-	wb->file_exporter = NULL;
+	workbook_set_file_exporter (wb, NULL);
 }
 
 void
-workbook_update_history (Workbook *wb)
+workbook_update_history (Workbook *wb, file_save_as_t type)
 {
 	g_return_if_fail (IS_WORKBOOK (wb));
 
-	if (wb->doc.uri && wb->file_format_level >= GO_FILE_FL_MANUAL_REMEMBER) {
-		const char *mimetype = wb->file_saver
-			? go_file_saver_get_mime_type (wb->file_saver)
-			: NULL;
-		gnm_app_history_add (wb->doc.uri, mimetype);
+	switch (type) {
+	case FILE_SAVE_AS_SAVE:
+		if (wb->doc.uri && wb->file_format_level >= GO_FILE_FL_MANUAL_REMEMBER) {
+			const char *mimetype = wb->file_saver
+				? go_file_saver_get_mime_type (wb->file_saver)
+				: NULL;
+			gnm_app_history_add (wb->doc.uri, mimetype);
+		}
+		break;
+	case FILE_SAVE_AS_EXPORT:
+	default:
+		if (wb->last_export_uri &&
+		    wb->file_export_format_level >= GO_FILE_FL_MANUAL_REMEMBER) {
+			const char *mimetype = wb->file_exporter
+				? go_file_saver_get_mime_type (wb->file_exporter)
+				: NULL;
+			gnm_app_history_add (wb->last_export_uri, mimetype);
+		}
+		break;
 	}
 }
 
@@ -103,6 +117,7 @@ workbook_dispose (GObject *wb_object)
 		workbook_set_saveinfo (wb, GO_FILE_FL_AUTO, NULL);
 	if (wb->file_exporter)
 		workbook_set_saveinfo (wb, GO_FILE_FL_WRITE_ONLY, NULL);
+	workbook_set_last_export_uri (wb, NULL);
 
 	/* Remove all the sheet controls to avoid displaying while we exit */
 	WORKBOOK_FOREACH_CONTROL (wb, view, control,
@@ -192,8 +207,10 @@ workbook_init (GObject *object)
 	workbook_set_1904 (wb, FALSE);
 
 	wb->file_format_level = GO_FILE_FL_NEW;
+	wb->file_export_format_level = GO_FILE_FL_NEW;
 	wb->file_saver        = NULL;
-	wb->file_exporter        = NULL;
+	wb->file_exporter     = NULL;
+	wb->last_export_uri   = NULL;
 
 	wb->during_destruction = FALSE;
 	wb->being_reordered    = FALSE;
@@ -386,8 +403,8 @@ workbook_new_with_sheets (int sheet_count)
 /**
  * workbook_set_saveinfo:
  * @wb: the workbook to modify
- * @name: the file name for this worksheet.
- * @level: the file format level
+ * @lev: the file format level
+ * @saver: the file saver.
  *
  * If level is sufficiently advanced assign the info.
  *
@@ -406,8 +423,7 @@ workbook_set_saveinfo (Workbook *wb, GOFileFormatLevel level, GOFileSaver *fs)
 		if (wb->file_exporter != NULL)
 			g_object_weak_unref (G_OBJECT (wb->file_exporter),
 					     (GWeakNotify) cb_exporter_finalize, wb);
-
-		wb->file_exporter = fs;
+		workbook_set_file_exporter (wb, fs);
 		if (fs != NULL)
 			g_object_weak_ref (G_OBJECT (fs),
 					   (GWeakNotify) cb_exporter_finalize, wb);
@@ -422,12 +438,20 @@ workbook_set_saveinfo (Workbook *wb, GOFileFormatLevel level, GOFileSaver *fs)
 					   (GWeakNotify) cb_saver_finalize, wb);
 	}
 
-	if (level == GO_FILE_FL_WRITE_ONLY)
+	if (level != GO_FILE_FL_AUTO) {
+		wb->file_export_format_level = level;
 		return FALSE;
+	}
 	wb->file_format_level = level;
 	return TRUE;
 }
 
+/**
+ * workbook_get_file_saver:
+ * @wb: #Workbook
+ *
+ * Returns: (transfer none): the saver for the Workbook.
+ **/
 GOFileSaver *
 workbook_get_file_saver (Workbook *wb)
 {
@@ -436,6 +460,12 @@ workbook_get_file_saver (Workbook *wb)
 	return wb->file_saver;
 }
 
+/**
+ * workbook_get_file_exporter:
+ * @wb: #Workbook
+ *
+ * Returns: (transfer none): the exporter for the Workbook.
+ **/
 GOFileSaver *
 workbook_get_file_exporter (Workbook *wb)
 {
@@ -444,13 +474,45 @@ workbook_get_file_exporter (Workbook *wb)
 	return wb->file_exporter;
 }
 
+/**
+ * workbook_get_last_export_uri:
+ * @wb: #Workbook
+ *
+ * Returns: (transfer none): the URI for export.
+ **/
+gchar const *
+workbook_get_last_export_uri (Workbook *wb)
+{
+	g_return_val_if_fail (IS_WORKBOOK (wb), NULL);
+
+	return wb->last_export_uri;
+}
+
+void
+workbook_set_file_exporter (Workbook *wb, GOFileSaver *fs)
+{
+	wb->file_exporter = fs;
+	WORKBOOK_FOREACH_CONTROL (wb, wbv, wbc,
+				  wb_control_menu_state_update (wbc, MS_FILE_EXPORT_IMPORT););
+}
+
+void
+workbook_set_last_export_uri (Workbook *wb, gchar *uri)
+{
+	g_free (wb->last_export_uri);
+	wb->last_export_uri = uri;
+	WORKBOOK_FOREACH_CONTROL (wb, wbv, wbc,
+				  wb_control_menu_state_update (wbc, MS_FILE_EXPORT_IMPORT););
+}
+
+
 /**
  * workbook_foreach_cell_in_range:
  *
  * @pos: The position the range is relative to.
  * @cell_range: A value containing a range;
- * @only_existing: if TRUE only existing cells are sent to the handler.
- * @handler: The operator to apply to each cell.
+ * @flags: if TRUE only existing cells are sent to the handler.
+ * @handler: (scope call): The operator to apply to each cell.
  * @closure: User data.
  *
  * The supplied value must be a cellrange.
@@ -515,6 +577,8 @@ workbook_foreach_cell_in_range (GnmEvalPos const *pos,
  *
  * Collects a GPtrArray of GnmEvalPos pointers for all cells in a workbook.
  * No particular order should be assumed.
+ *
+ * Returns: (element-type GnmEvalPos) (transfer full): the cells array
  */
 GPtrArray *
 workbook_cells (Workbook *wb, gboolean comments, GnmSheetVisibility vis)
@@ -577,6 +641,15 @@ workbook_optimize_style (Workbook *wb)
 	});
 }
 
+/**
+ * workbook_foreach_name:
+ *
+ * @wb: #Workbook
+ * @globals_only: whether to apply only to global names.
+ * @func: (scope call): The operator to apply to each cell.
+ * @data: User data.
+ *
+ **/
 void
 workbook_foreach_name (Workbook const *wb, gboolean globals_only,
 		       GHFunc func, gpointer data)
@@ -680,8 +753,12 @@ workbook_detach_view (WorkbookView *wbv)
 /*****************************************************************************/
 
 /**
- * workbook_sheets: Get an ordered list of the sheets in the workbook
- *                  The caller is required to free the list.
+ * workbook_sheets:
+ * @wb: #Workbook
+ *
+ * Get an ordered list of the sheets in the workbook
+ * The caller is required to free the list.
+ * Returns: (element-type Sheet) (transfer container): the sheets list.
  */
 GSList *
 workbook_sheets (Workbook const *wb)
@@ -1120,13 +1197,13 @@ workbook_sheet_get_free_name (Workbook *wb,
 /**
  * workbook_sheet_rename:
  * @wb:          workbook to look for
- * @sheet_indices:   list of sheet indices (ignore -1)
- * @new_names:   list of new names
+ * @sheet_indices: (element-type void):  list of sheet indices (ignore -1)
+ * @new_names: (element-type char):  list of new names
  *
  * Adjusts the names of the sheets. We assume that everything is
  * valid. If in doubt call workbook_sheet_reorder_check first.
  *
- * Returns FALSE when it was successful
+ * Returns: FALSE when it was successful
  **/
 gboolean
 workbook_sheet_rename (Workbook *wb,
@@ -1165,9 +1242,9 @@ workbook_sheet_rename (Workbook *wb,
  * workbook_find_command:
  * @wb: #Workbook
  * @is_undo: undo vs redo
- * @key: command
+ * @cmd: command
  *
- * returns the 1 based index of the @key command, or 0 if it is not found
+ * Returns: the 1 based index of the @key command, or 0 if it is not found
  **/
 unsigned
 workbook_find_command (Workbook *wb, gboolean is_undo, gpointer cmd)
@@ -1188,7 +1265,7 @@ workbook_find_command (Workbook *wb, gboolean is_undo, gpointer cmd)
 /**
  * workbook_sheet_reorder:
  * @wb:          workbook to look for
- * @new_order:   list of sheets
+ * @new_order: (element-type Sheet):  list of sheets
  *
  * Adjusts the order of the sheets.
  *
@@ -1256,6 +1333,12 @@ workbook_set_1904 (Workbook *wb, gboolean base1904)
 	workbook_set_date_conv (wb, date_conv);
 }
 
+/**
+ * workbook_get_sheet_size:
+ * @wb: #Workbook
+ *
+ * Returns: (transfer none): the current sheet size for @wb.
+ **/
 GnmSheetSize const *
 workbook_get_sheet_size (Workbook const *wb)
 {
@@ -1281,6 +1364,7 @@ struct _WorkbookSheetState {
 	GSList *properties;
 	int n_sheets;
 	WorkbookSheetStateSheet *sheets;
+	unsigned ref_count;
 };
 
 
@@ -1298,6 +1382,7 @@ workbook_sheet_state_new (const Workbook *wb)
 		wsss->sheet = g_object_ref (workbook_sheet_by_index (wb, i));
 		wsss->properties = go_object_properties_collect (G_OBJECT (wsss->sheet));
 	}
+	wss->ref_count = 1;
 	return wss;
 }
 
@@ -1306,6 +1391,9 @@ workbook_sheet_state_free (WorkbookSheetState *wss)
 {
 	int i;
 
+	if (!wss || wss->ref_count-- > 1)
+		return;
+
 	go_object_properties_free (wss->properties);
 
 	for (i = 0; i < wss->n_sheets; i++) {
@@ -1317,6 +1405,26 @@ workbook_sheet_state_free (WorkbookSheetState *wss)
 	g_free (wss);
 }
 
+static WorkbookSheetState *
+workbook_sheet_state_ref (WorkbookSheetState *wss)
+{
+	wss->ref_count++;
+	return wss;
+}
+
+GType
+workbook_sheet_state_get_type (void)
+{
+	static GType t = 0;
+
+	if (t == 0) {
+		t = g_boxed_type_register_static ("WorkbookSheetState",
+			 (GBoxedCopyFunc)workbook_sheet_state_ref,
+			 (GBoxedFreeFunc)workbook_sheet_state_free);
+	}
+	return t;
+}
+
 void
 workbook_sheet_state_restore (Workbook *wb, const WorkbookSheetState *wss)
 {
